
/**
 * \file SDTest_1.c
 * \brief AVR MEGARF SPI Slave example
 *
 * create a .txt File and write text into the file<br>
 * project: EVALUATIONKIT
 *
 */

/** \mainpage
 * \section intro Introduction
 * This test project demonstrates the use of ElmChaNs FatFs to write something on SD card.<br>
 * \section files Main Files
 * - SDTest_1.c: the test application.
 * - sdio.c: configuration SPI functions to connect FatFs with the hardware interface
 * - ffconf.h: configuration-file of FatFs 
 *
 * \section deviceinfo Device Info
 * - Controler: AVR ATMEGA1284-P
 * - Testboard: Pollin Atmel Valuationboard Ver. 2.0 -> modified to 3,3V supply<br>
 * This testboard was build up with some changes. In the following picture you see the whole set-up<br>
 * \image html Schaltung.jpg
 * For the SD Card you need 3.3V. Because the voltage regulator of the Valuationboard was 5V we changed it with an adapterboard to an 3.3V<br>
 * \image html 3V3Regler.jpg
 * - Adapterboard: SD Card test board + test board for microC SPI connection (with ISP - 1k Resistors) <br>
 * To guarantee the SPI connection does not affect the ISP programmer, there are 1k resistors between forward the SCK, MISO and MOSI of the SD Card.<br>
 * \image html sd-spi.jpg
 *<br>
 *
 */


/*
 * SDTest_1.c
 *
 * Created: 12.08.2014 11:13:23
 *  Author: ED1983
 */ 



#ifndef F_CPU
	#define F_CPU 16000000UL ///< Oscillator frequency: 16MHz  (unsigned long)
#endif

#include <avr/io.h>
#include <avr/interrupt.h>
#include "diskio.h"
#include "ff.h"
#include <util/delay.h>
#include "main.h"

volatile UINT Timer;		/* Performance timer (100Hz increment) */


FATFS fs;  ///<File system object
FIL fin;   ///<File object


/*  
* \brief main function 
* \param
* \return
*
* I/O-Configuration for Timerinterrupt<br>
* SD Intitalisation and write text into a new file
*
* \date 12.08.2014 - First implementation
*/
int main(void)
{
	UINT bw; //Number of bytes to write
	//FRESULT res; //Function output -> Error Code
	int res;
	 // Timer 0 konfigurieren
	DDRD  = 0xFF;
	SETBIT(PORTD,6);
	SETBIT(PORTD,5);
	 // Overflow Interrupt erlauben
	OCR2A = 156-1;//(F_CPU / 1024 / 1000) - 1; //Timer2: 100Hz interval (OC2)  
	TCCR2A = _BV(WGM21);  //Normal port operation
	TCCR2B = _BV(CS22) | _BV(CS21) |_BV(CS20);  //clkT2S/1024 (From prescaler)
	TIMSK2 = _BV(OCIE2A);   // Enable TC2.oc interrupt
	sei(); 
	
	//initialize SD Card for SPI mode
	res=disk_initialize(0);
		
		
	if ( res== STA_NOINIT) {
		while(1){

		}
	}
	
	
	//mount SD Card for Fat Operations
	res = f_mount(&fs, "", 0);
	if (res == FR_OK) {	
		
		res = f_open(&fin, "poem.txt",  (FA_WRITE | FA_CREATE_NEW) );//open/create File
		

			if (res == FR_OK) {

			f_write(&fin, "It works!\r\n", 11, &bw); //write into file
			f_close(&fin); //close file and synchronize data
			if(bw==11){
				
			}
		}
		
	}

	while(1)
	{
	}
}

ISR(TIMER2_COMPA_vect) {
	Timer++;
	void disk_timerproc();
}

DWORD get_fattime (void)
{
	/* Pack date and time into a DWORD variable */
	return     ((DWORD)(2013 - 1980) << 25)
	| ((DWORD)3 << 21)
	| ((DWORD)23 << 16)
	| ((DWORD)12 << 11)
	| ((DWORD)0 << 5)
	| ((DWORD)0 >> 1);
}